var p = require('phin').promisified

function VimeoAnalytics(player, authToken) {
  this.player = player
  this.authToken = authToken
  this.metadata = {
    content: {},
    playback: { videoPlayer: 'Vimeo' }
  }
  this.mostRecentHeartbeat = 0
  this.isPaused = false
}

VimeoAnalytics.prototype.initialize = function() {
  var events = {
    loaded: this.retrieveMetadata,
    play: this.trackPlay,
    pause: this.trackPause,
    ended: this.trackEnded,
    timeupdate: this.trackHeartbeat
  }

  for (event in events) {
    this.registerHandler(event, events[event])
  }
}

VimeoAnalytics.prototype.registerHandler = function(event, handler) {
  this.player.on(event, data => {
    this.updateMetadata(data)
    handler.call(this, data)
  })
}

VimeoAnalytics.prototype.trackPlay = function() {
  if (this.isPaused) {
    window.analytics.track('Video Playback Resumed', this.metadata.playback)
    this.isPaused = false
  } else {
    window.analytics.track('Video Playback Started', this.metadata.playback)
    window.analytics.track('Video Content Started', this.metadata.content)
  }
}

VimeoAnalytics.prototype.trackEnded = function() {
  window.analytics.track('Video Playback Completed', this.metadata.playback)
  window.analytics.track('Video Content Completed', this.metadata.content)
}

VimeoAnalytics.prototype.trackHeartbeat = function() {
  var mostRecentHeartbeat = this.mostRecentHeartbeat
  var currentPosition = this.metadata.playback.position
  if (
    currentPosition !== mostRecentHeartbeat &&
    currentPosition - mostRecentHeartbeat >= 10
  ) {
    window.analytics.track('Video Content Playing', this.metadata.content)
    this.mostRecentHeartbeat = Math.floor(currentPosition)
  }
}

VimeoAnalytics.prototype.trackPause = function() {
  this.isPaused = true
  window.analytics.track('Video Playback Paused', this.metadata.playback)
}

VimeoAnalytics.prototype.retrieveMetadata = function(data) {
  var self = this
  return new Promise(function(resolve, reject) {
    var videoId = data.id
    var options = {
      url: 'https://api.vimeo.com/videos/' + videoId,
      headers: {
        Authorization: 'Bearer ' + self.authToken
      }
    }

    p(options).then(function(res) {
      res = JSON.parse(res.body)
      if (res.error) {
        return reject(res)
      }
      self.metadata.content.title = res.name
      self.metadata.content.description = res.description
      self.metadata.content.publisher = res.user.name

      self.metadata.playback.position = 0
      self.metadata.playback.totalLength = res.duration
      return resolve()
    }).catch(reject)
  })
}

VimeoAnalytics.prototype.updateMetadata = function(data) {
  var self = this
  return new Promise(function(resolve, reject) {
    var volume = self.player.getVolume().then(function(volume) {
      if (volume) self.metadata.playback.sound = volume * 100
      self.metadata.playback.position = data.seconds
    })
    .then(resolve)
    .catch(reject)
  })
}

module.exports = VimeoAnalytics