var VimeoAnalytics = require('../src')
var assert = require('assert')
var sandbox = sinon.createSandbox()

describe('VimeoAnalytics', () => {
  var player
  var vimeoAnalytics

  beforeEach(function() {
    window.analytics = {
      track: sandbox.spy()
    }

    var html =
      '<iframe id="video-player" src="https://player.vimeo.com/video/76979871" width="640" height="360" frameborder="0" webkitallowfullscreen mozallowfullscreen allowfullscreen></iframe>'
    document.body.insertAdjacentHTML('afterbegin', html)
    var iframe = document.querySelector('iframe')
    player = new Vimeo.Player(iframe, { autoplay: true, muted: true })
    vimeoAnalytics = new VimeoAnalytics(
      player,
      'b203643ead884f6b47b193c88f328c4a'
    )
    vimeoAnalytics.track = sandbox.spy()
    vimeoAnalytics.initialize()
  })

  afterEach(function() {
    return player.unload().then(function() {
      clearEnv('video-player')
      sandbox.reset()
    })
  })

  describe('registerHandler', function() {
    it('should call player.on for any event with the given handler', function() {
      sandbox.spy(player, 'on')
      vimeoAnalytics.registerHandler('play', function() {})
      player.on.should.have.been.calledOnce
      player.on.should.have.been.calledWithMatch(
        sinon.match('play'),
        sinon.match.func
      )
    })
  })

  describe('initialize()', function() {
    it('should initiate all supported event handlers', function() {
      sandbox.spy(vimeoAnalytics, 'registerHandler')

      var events = {
        loaded: vimeoAnalytics.retrieveMetadata,
        play: vimeoAnalytics.trackPlay,
        pause: vimeoAnalytics.trackPause,
        ended: vimeoAnalytics.trackEnded,
        timeupdate: vimeoAnalytics.trackHeartbeat
      }

      vimeoAnalytics.initialize()

      for (var event in events) {
        vimeoAnalytics.registerHandler.should.have.been.calledWith(
          event,
          events[event]
        )
      }
    })
  })

  describe('Event Handlers', function() {
    describe('retrieveMetadata()', function() {
      it('should successfully retrieve metadata for a given video', function() {
        return vimeoAnalytics
          .retrieveMetadata({ id: 76979871 })
          .then(function() {
            assert.equal(
              vimeoAnalytics.metadata.content.title,
              'The New Vimeo Player (You Know, For Videos)'
            )
            assert(
              typeof vimeoAnalytics.metadata.content.description === 'string'
            )
            assert.equal(
              vimeoAnalytics.metadata.content.publisher,
              'Vimeo Staff'
            )
            assert.equal(vimeoAnalytics.metadata.playback.videoPlayer, 'Vimeo')
            assert.equal(vimeoAnalytics.metadata.playback.position, 0)
            assert.equal(vimeoAnalytics.metadata.playback.totalLength, 62)
          })
      })
    })

    describe('trackPlay()', function() {
      it('should track play events as Video Playback Started events', function() {
        vimeoAnalytics.trackPlay()
        window.analytics.track.firstCall.should.have.been.calledWith(
          'Video Playback Started',
          vimeoAnalytics.metadata.playback
        )
      })

      it('should not track Video Playback Started events if playback is paused', function() {
        vimeoAnalytics.isPaused = true
        vimeoAnalytics.trackPlay()
        window.analytics.track.should.not.have.been.calledWith(
          'Video Playback Started'
        )
      })

      it('should track play events as Video Playback Resumed events if the video was previously paused', function() {
        vimeoAnalytics.isPaused = true
        vimeoAnalytics.trackPlay()
        window.analytics.track.should.have.been.calledWith(
          'Video Playback Resumed',
          vimeoAnalytics.metadata.playback
        )
      })

      it('should track play events as Video Content Started events', function() {
        vimeoAnalytics.trackPlay()
        window.analytics.track.should.have.been.calledWith(
          'Video Content Started',
          vimeoAnalytics.metadata.content
        )
      })
    })

    describe('trackPause()', function() {
      var pauseData = {
        duration: 61.857,
        percent: 0.049,
        seconds: 3.034
      }

      it('should set isPaused to true', function() {
        return vimeoAnalytics.updateMetadata(pauseData).then(function() {
          vimeoAnalytics.trackPause()
          assert(vimeoAnalytics.isPaused === true)
        })
      })

      it('should track pause events as Video Playback Paused events', function() {
        return vimeoAnalytics.updateMetadata(pauseData).then(function() {
          vimeoAnalytics.trackPause()
          window.analytics.track.should.have.been.calledOnce
          window.analytics.track.should.have.been.calledWith(
            'Video Playback Paused',
            vimeoAnalytics.metadata.playback
          )
        })
      })
    })

    describe('trackHeartbeat()', function() {
      var getData = function(seconds) {
        seconds = seconds || 0
        var duration = 61.857
        return {
          percent: seconds / duration,
          duration,
          seconds
        }
      }

      it('should not track a heartbeat event if it has not been 10 seconds since the last hearbeat', function() {
        vimeoAnalytics.mostRecentHeartbeat = 10
        vimeoAnalytics.trackHeartbeat(getData(15.02))
        window.analytics.track.should.not.have.been.called
      })

      it('should track a single heartbeat event at 10 second intervals', function() {
        vimeoAnalytics.mostRecentHeartbeat = 10
        return vimeoAnalytics.updateMetadata(getData(20.03)).then(function() {
          vimeoAnalytics.trackHeartbeat()
          vimeoAnalytics.updateMetadata(getData(22.07))
          vimeoAnalytics.trackHeartbeat()
          window.analytics.track.should.have.been.calledOnce
          window.analytics.track.should.have.been.calledWith(
            'Video Content Playing',
            vimeoAnalytics.metadata.content
          )
        })
      })

      it('should update the mostRecentHeartbeat flag when a new heartbeat is triggered', function() {
        vimeoAnalytics.mostRecentHeartbeat = 20
        return vimeoAnalytics.updateMetadata(getData(30)).then(function() {
          vimeoAnalytics.trackHeartbeat()
          assert.equal(vimeoAnalytics.mostRecentHeartbeat, 30)
        })
      })
    })

    describe('trackEnded()', function() {
      it('should track end events as Video Playback Completed events', function() {
        vimeoAnalytics.trackEnded()
        window.analytics.track.should.have.been.calledWith(
          'Video Playback Completed',
          vimeoAnalytics.metadata.playback
        )
      })

      it('should track end events as Video Content Completed events', async function() {
        vimeoAnalytics.trackEnded()
        var contentCompletedSpy = window.analytics.track.withArgs(
          'Video Content Completed',
          vimeoAnalytics.metadata.content
        )
        contentCompletedSpy.should.have.been.calledOnce
      })
    })
  })
})

function clearEnv(id) {
  document.getElementById(id).remove()
}
