var p = require('phin').promisified

const SEEK_THRESHOLD = 2000

function YouTube(player, apiKey) {
  this.player = player
  this.apiKey = apiKey
  this.playerLoaded = false
  this.playbackStarted = false
  this.contentStarted = false
  this.isPaused = false
  this.isBuffering = false
  this.isSeeking = false
  this.lastRecordedTime = { // updated every event
    timeReported: Date.now(),
    timeElapsed: 0.0
  }
  this.metadata = []
  this.playlistIndex = 0
}

YouTube.prototype.initialize = function() {
  // Youtube API requires listeners to exist as top-level props on global object
  global.segmentYoutubeOnStateChange = this.onPlayerStateChange.bind(this)
  global.segmentYoutubeOnReady = this.onPlayerReady.bind(this)

  var player = this.player

  player.addEventListener('onReady', 'segmentYoutubeOnReady')
  player.addEventListener('onStateChange', 'segmentYoutubeOnStateChange')
}

YouTube.prototype.onPlayerReady = function(event) {  // this fires when the player html element loads
  if (this.player.getPlaylistId()) {
    this.player.cuePlaylist({
      listType: 'playlist',
      list: this.player.getPlaylistId()
    })
  } else {
    var videoData = this.player.getVideoData()
    this.player.cueVideoById(videoData.video_id)
  }
}

// yt reports events via state changes in the player
YouTube.prototype.onPlayerStateChange = function(event) {
  const currentTime = this.player.getCurrentTime()

  if (this.metadata[this.playlistIndex]) {
    this.metadata[this.playlistIndex].playback.position = this.metadata[this.playlistIndex].content.position = currentTime
    this.metadata[this.playlistIndex].playback.quality = this.player.getPlaybackQuality()
    this.metadata[this.playlistIndex].playback.sound = this.player.getVolume()
  }

  switch (event.data) {
    case -1:  // this fires when a video or playlist has loaded
      if (this.playerLoaded) break
      this.retrieveMetadata()
      this.playerLoaded = true
      break

    case YT.PlayerState.BUFFERING:
      this.handleBuffer()
      break

    case YT.PlayerState.PLAYING:
      this.handlePlay()
      break

    case YT.PlayerState.PAUSED:
      this.handlePause()
      break

    case YT.PlayerState.ENDED:
      this.handleEnd()
      break
  }

  this.lastRecordedTime = {
    timeReported: Date.now(),
    timeElapsed: this.player.getCurrentTime()*1000.0
  }
}

YouTube.prototype.retrieveMetadata = function() {
  var self = this
  return new Promise(function(resolve, reject) {
    var videoData = self.player.getVideoData()
    var playlist = self.player.getPlaylist() || [videoData.video_id]
    var videoIds = playlist.join()
    var url = 'https://www.googleapis.com/youtube/v3/videos?id=' + videoIds + '&part=snippet,contentDetails&key=' + self.apiKey

    p(url).then(function(res) {
      res = JSON.parse(res.body)
      var total_length = 0
      for (var i=0; i<playlist.length; i++) {
        var videoInfo = res.items[i]
        self.metadata.push({
          content: {
            title: videoInfo.snippet.title,
            description: videoInfo.snippet.description,
            keywords: videoInfo.snippet.tags,
            channel: videoInfo.snippet.channelTitle,
            airdate: videoInfo.snippet.publishedAt,
          }
        })
        total_length += YTDurationToSeconds(videoInfo.contentDetails.duration)
      }

      for (var i=0; i<playlist.length; i++) {
        self.metadata[i].playback = {
          total_length,
          video_player: 'youtube'
        }
      }
      return resolve()
    }).catch(reject)
  })
}

YouTube.prototype.handleBuffer = function() {
  var seekDetected = this.determineSeek()

  if (!this.playbackStarted) {
    this.playbackStarted = true
    global.analytics.track('Video Playback Started', this.metadata[this.playlistIndex].playback)
  }

  // user used keyboard to seek or seeked while video was paused
  if (seekDetected && !this.isSeeking) {
    this.isSeeking = true
    global.analytics.track('Video Playback Seek Started', this.metadata[this.playlistIndex].playback)
  }

  // state changing to BUFFERING denotes seeking has completed
  if (this.isSeeking) {
    global.analytics.track('Video Playback Seek Completed', this.metadata[this.playlistIndex].playback)
    this.isSeeking = false
  }

  // user clicked next video button
  if (this.player.getCurrentTime() === 0 && this.player.getPlaylistIndex() !== this.playlistIndex) {
    this.contentStarted = false
    if (this.playlistIndex === this.metadata.length - 1 && this.player.getPlaylistIndex() === 0) { // user skipped to end of last video in playlist
      global.analytics.track('Video Playback Completed', this.metadata[this.player.getPlaylistIndex()].playback)
      global.analytics.track('Video Playback Started', this.metadata[this.player.getPlaylistIndex()].playback)  // playlist automatically starts from beginning
    }
  }

  global.analytics.track('Video Playback Buffer Started', this.metadata[this.playlistIndex].playback)
  this.isBuffering = true
}

YouTube.prototype.handlePlay = function() {
  if (!this.contentStarted) {
    this.playlistIndex = this.player.getPlaylistIndex()  // will return -1 if the player has a singular video instead of a playlist
    if (this.playlistIndex === -1) this.playlistIndex = 0
    global.analytics.track('Video Content Started', this.metadata[this.playlistIndex].content)
    this.contentStarted = true
  }

  if (this.isBuffering) {
    global.analytics.track('Video Playback Buffer Completed', this.metadata[this.playlistIndex].playback)
    this.isBuffering = false
  }

  if (this.isPaused) {
    global.analytics.track('Video Playback Resumed', this.metadata[this.playlistIndex].playback)
    this.isPaused = false
  }
}

YouTube.prototype.handlePause = function() {
  var seekDetected = this.determineSeek()
  // user seeked while video was paused, it buffered, and then finished buffering
  if (this.isBuffering) {
    global.analytics.track('Video Playback Buffer Completed', this.metadata[this.playlistIndex].playback)
    this.isBuffering = false
  }

  if (!this.isPaused) {
    // user seeked while video was playing
    if (seekDetected) {
      global.analytics.track('Video Playback Seek Started', this.metadata[this.playlistIndex].playback)
      this.isSeeking = true
    }
    // user clicked pause while video was playing
    else {
      global.analytics.track('Video Playback Paused', this.metadata[this.playlistIndex].playback)
      this.isPaused = true
    }
  }
}

YouTube.prototype.handleEnd = function() {
  global.analytics.track('Video Content Completed', this.metadata[this.playlistIndex].content)
  this.contentStarted = false

  var playlistIndex = this.player.getPlaylistIndex()
  var playlist = this.player.getPlaylist()

  if ((playlist && playlistIndex === playlist.length-1) || (playlistIndex === -1)) {
    global.analytics.track('Video Playback Completed', this.metadata[this.playlistIndex].playback)
    this.playbackStarted = false
  }
}

  // yt doesn't natively track seeking so we have to manually calculate whether a seek has occurred based on expected vs actual event timestamps
YouTube.prototype.determineSeek = function() {
  var expectedTimeElapsed = (this.isPaused || this.isBuffering) ? 0 : Date.now() - this.lastRecordedTime.timeReported
  var actualTimeElapsed = this.player.getCurrentTime()*1000.0 - this.lastRecordedTime.timeElapsed

  return Math.abs(expectedTimeElapsed - actualTimeElapsed) > SEEK_THRESHOLD // if the diff btwn the 2 is > the threshold we can reasonably assume a seek has occurred
}

// convert from ISO 8601 to seconds
function YTDurationToSeconds(duration) {
  var match = duration.match(/PT(\d+H)?(\d+M)?(\d+S)?/);

  match = match.slice(1).map(function(x) {
    if (x != null) {
        return x.replace(/\D/, '');
    }
  });

  var hours = (parseInt(match[0]) || 0);
  var minutes = (parseInt(match[1]) || 0);
  var seconds = (parseInt(match[2]) || 0);

  return hours * 3600 + minutes * 60 + seconds;
}

module.exports = YouTube