var YoutubeAnalytics = require('../src')
var assert = require('assert')
var sinon = require('sinon')
var fetchMock = require('fetch-mock')

var SEEK_THRESHOLD = 2000

var sampleSingleVideoApiOutput = {
  body: {
    items: [
      {
        kind: 'youtube#video',
        etag: '"XI7nbFXulYBIpL0ayR_gDh3eu1k/l8T3Ps_vgKQV_OC2bHkLGQ44KAk"',
        id: 'M7lc1UVf-VE',
        snippet: {
          publishedAt: '2013-04-10T17:25:04.000Z',
          channelId: 'UC_x5XG1OV2P6uZZ5FSM9Ttw',
          title: 'YouTube Developers Live: Embedded Web Player Customization',
          description:
            "On this week's show, Jeff Posnick covers everything you need to know about using player parameters to customize the YouTube iframe-embedded player.",
          channelTitle: 'Google Developers',
          tags: [
            'youtubedataapi',
            'youtube',
            'youtubeplayerapi',
            'youtubeapi',
            'video',
            'Mountain View',
            'gdl'
          ],
          categoryId: '28',
          liveBroadcastContent: 'none',
          localized: {
            title: 'YouTube Developers Live: Embedded Web Player Customization',
            description:
              "On this week's show, Jeff Posnick covers everything you need to know about using player parameters to customize the YouTube iframe-embedded player."
          },
          defaultAudioLanguage: 'en'
        },
        contentDetails: {
          duration: 'PT22M24S',
          dimension: '2d',
          definition: 'hd',
          caption: 'true',
          licensedContent: false,
          projection: 'rectangular'
        }
      }
    ]
  }
}

var samplePlaylistApiOutput = {
  body: {
    items: [
      {
        kind: 'youtube#video',
        etag: '"XI7nbFXulYBIpL0ayR_gDh3eu1k/l8T3Ps_vgKQV_OC2bHkLGQ44KAk"',
        id: 'M7lc1UVf-VE',
        snippet: {
          publishedAt: '2013-04-10T17:25:04.000Z',
          channelId: 'UC_x5XG1OV2P6uZZ5FSM9Ttw',
          title: 'YouTube Developers Live: Embedded Web Player Customization',
          description:
            "On this week's show, Jeff Posnick covers everything you need to know about using player parameters to customize the YouTube iframe-embedded player.",
          channelTitle: 'Google Developers',
          tags: [
            'youtubedataapi',
            'youtube',
            'youtubeplayerapi',
            'youtubeapi',
            'video',
            'Mountain View',
            'gdl'
          ],
          categoryId: '28',
          liveBroadcastContent: 'none',
          localized: {
            title: 'YouTube Developers Live: Embedded Web Player Customization',
            description:
              "On this week's show, Jeff Posnick covers everything you need to know about using player parameters to customize the YouTube iframe-embedded player."
          },
          defaultAudioLanguage: 'en'
        },
        contentDetails: {
          duration: 'PT22M24S',
          dimension: '2d',
          definition: 'hd',
          caption: 'true',
          licensedContent: false,
          projection: 'rectangular'
        }
      },
      {
        kind: 'youtube#video',
        etag: '"XI7nbFXulYBIpL0ayR_gDh3eu1k/WRztWOH0sUonKk2lGPf4GfmmTjk"',
        id: 'izoct69J4v8',
        snippet: {
          publishedAt: '2017-10-26T21:49:55.000Z',
          channelId: 'UCTJmlYi-adThOQaXuuNYsgQ',
          title: 'Introducing Personas, by Segment',
          description:
            'Derrick thinks Kicks Central controls the weather. Turns out, its just the corporate office using Personas by Segment to send the right offers to the right people, at exactly the right time. Learn more at segment.com/personas.',
          channelTitle: 'Segment',
          tags: [
            'segment',
            'personas',
            'identity resolution',
            'audience building',
            'custom segments',
            'marketing automation',
            'api'
          ],
          categoryId: '27',
          liveBroadcastContent: 'none',
          localized: {
            title: 'Introducing Personas, by Segment',
            description:
              'Derrick thinks Kicks Central controls the weather. Turns out, its just the corporate office using Personas by Segment to send the right offers to the right people, at exactly the right time. Learn more at segment.com/personas.'
          }
        },
        contentDetails: {
          duration: 'PT1M30S',
          dimension: '2d',
          definition: 'hd',
          caption: 'false',
          licensedContent: false,
          projection: 'rectangular'
        }
      }
    ]
  }
}

window.YT = {
  PlayerState: {
    ENDED: 0,
    PLAYING: 1,
    PAUSED: 2,
    BUFFERING: 3
  }
}

describe('YoutubeAnalytics', function() {
  var youtubeAnalytics
  var playerTime
  var playbackQuality
  var volume
  var playlistIndex
  var player
  var playlistId
  var videoId = 'M7lc1UVf'
  var playlist

  beforeEach(function() {
    playerTime = 0
    playlistId = null
    volume = 0
    playbackQuality = 0

    // stub out player methods
    player = {
      addEventListener: function () {},
      getVideoData: function() {
        return { video_id: videoId}
      },
      getPlaylist: function() { return playlist },
      getCurrentTime: function() { return playerTime },
      getPlaybackQuality: function() { return playbackQuality },
      getVolume: function() { return volume },
      getPlaylistIndex: function() { return isNaN(playlistIndex) ? -1 : playlistIndex },
      getPlaylistId: function() { return playlistId },
      cuePlaylist: function() {},
      cueVideoById: function() {}
    }

    window.analytics = {
      track: sinon.spy()
    }
    youtubeAnalytics = new YoutubeAnalytics(
      player,
      'AIzaSyDxtBpYa_IAWudnLW0P79sBF-cIzBUOUpQ'
    )
  })

  describe('initialize()', function() {
    it('should put listeners on the window and add the listeners to the player object', function() {
      var stub = sinon.stub(player, 'addEventListener')

      youtubeAnalytics.initialize()

      assert(global.segmentYoutubeOnStateChange)
      assert(global.segmentYoutubeOnReady)
      sinon.assert.calledWith(
        stub,
        'onStateChange',
        'segmentYoutubeOnStateChange'
      )
      sinon.assert.calledWith(stub, 'onReady', 'segmentYoutubeOnReady')
    })
  })

  describe('onPlayerReady', function() {
    it('should cue a playlist if a playlist id has been provided', function() {
      playlistId = '1234'
      var stub = sinon.stub(youtubeAnalytics.player, 'cuePlaylist')
      youtubeAnalytics.onPlayerReady()

      sinon.assert.calledWith(stub, {
        listType: 'playlist',
        list: playlistId
      })
    })

    it('should cue a single video if no playlist id has been provided', function() {
      var stub = sinon.stub(youtubeAnalytics.player, 'cueVideoById')
      youtubeAnalytics.onPlayerReady()

      sinon.assert.calledWith(stub, videoId)
    })
  })

  describe('Metadata Retrieval', function() {
    afterEach(function() {
      fetchMock.restore()
    })

    it('should successfully retrieve metadata for a given video', function() {
      fetchMock.mock(
        'begin:https://www.googleapis.com',
        sampleSingleVideoApiOutput
      )
      return youtubeAnalytics.retrieveMetadata().then(function() {
        assert.deepEqual(youtubeAnalytics.metadata, [
          {
            content: {
              title:
                'YouTube Developers Live: Embedded Web Player Customization',
              description:
                "On this week's show, Jeff Posnick covers everything you need to know about using player parameters to customize the YouTube iframe-embedded player.",
              keywords: [
                'youtubedataapi',
                'youtube',
                'youtubeplayerapi',
                'youtubeapi',
                'video',
                'Mountain View',
                'gdl'
              ],
              channel: 'Google Developers',
              airdate: '2013-04-10T17:25:04.000Z'
            },
            playback: { total_length: 1344, video_player: 'youtube' }
          }
        ])
      })
    })

    it('should successfully retrieve metadata for a playlist of multiple videos', function() {
      fetchMock.mock(
        'begin:https://www.googleapis.com',
        samplePlaylistApiOutput
      )
      youtubeAnalytics.player.getPlaylist = function() {
        return ['M7lc1UVf-VE', 'izoct69J4v8']
      }
      youtubeAnalytics.retrieveMetadata().then(function() {
        assert.deepEqual(youtubeAnalytics.metadata, [
          {
            content: {
              title:
                'YouTube Developers Live: Embedded Web Player Customization',
              description:
                "On this week's show, Jeff Posnick covers everything you need to know about using player parameters to customize the YouTube iframe-embedded player.",
              keywords: [
                'youtubedataapi',
                'youtube',
                'youtubeplayerapi',
                'youtubeapi',
                'video',
                'Mountain View',
                'gdl'
              ],
              channel: 'Google Developers',
              airdate: '2013-04-10T17:25:04.000Z'
            },
            playback: { total_length: 1434, video_player: 'youtube' }
          },
          {
            content: {
              title: 'Introducing Personas, by Segment',
              description:
                'Derrick thinks Kicks Central controls the weather. Turns out, its just the corporate office using Personas by Segment to send the right offers to the right people, at exactly the right time. Learn more at segment.com/personas.',
              keywords: [
                'segment',
                'personas',
                'identity resolution',
                'audience building',
                'custom segments',
                'marketing automation',
                'api'
              ],
              channel: 'Segment',
              airdate: '2017-10-26T21:49:55.000Z'
            },
            playback: { total_length: 1434, video_player: 'youtube' }
          }
        ])
      })
    })

    it('should not fail on API request failure', function() {
      fetchMock.mock('begin:https://www.googleapis.com', 400)
      youtubeAnalytics.retrieveMetadata().then(function() {
        assert.deepEqual(youtubeAnalytics.metadata, [
          { playback: { video_player: 'youtube' }, content: {} }
        ])
      })
    })
  })

  describe('onPlayerStateChange', function() {
    beforeEach(function() {
      youtubeAnalytics.initialize()
      youtubeAnalytics.metadata = [{ playback: {}, content: {} }]
    })
    it('should update playback metadata', function() {
      playerTime = 4
      playbackQuality = 3
      volume = 2
      window.segmentYoutubeOnStateChange({})

      assert.deepEqual(youtubeAnalytics.metadata[0].playback, {
        position: playerTime,
        quality: playbackQuality,
        sound: volume
      })
    })

    it('should update "last recorded time"', function() {
      playerTime = 4

      var timeBeforeFunctionRun = Date.now()

      window.segmentYoutubeOnStateChange({})

      // impossible to know what Date.now was at the time the var was set so settle for checking that the updated value
      // is for a later date than Date.now before the function was run
      assert(
        youtubeAnalytics.lastRecordedTime.timeReported >= timeBeforeFunctionRun
      )
      assert.equal(
        youtubeAnalytics.lastRecordedTime.timeElapsed,
        playerTime * 1000.0
      )
    })
  })

  describe('Video Events', function() {
    var stub
    var playback
    var content
    beforeEach(function() {
      youtubeAnalytics.initialize()
      youtubeAnalytics.metadata = [{ playback: {}, content: {} }]
      stub = window.analytics.track
      playback = {
        position: playerTime,
        quality: playbackQuality,
        sound: volume
      }
      content = {
        title: 'YouTube Developers Live: Embedded Web Player Customization',
        description:
          "On this week's show, Jeff Posnick covers everything you need to know about using player parameters to customize the YouTube iframe-embedded player.",
        keywords: [
          'youtubedataapi',
          'youtube',
          'youtubeplayerapi',
          'youtubeapi',
          'video',
          'Mountain View',
          'gdl'
        ],
        channel: 'Google Developers',
        airdate: '2013-04-10T17:25:04.000Z'
      }
      youtubeAnalytics.metadata = [{ playback, content }]
    })
    describe('Singular Video Playback', function() {
      describe('YT buffering events', function() {
        it('should trigger "Playback Started","Playback Buffer Started" if video buffering for the first time', async function() {
          window.segmentYoutubeOnStateChange({ data: YT.PlayerState.BUFFERING })

          assert(youtubeAnalytics.isBuffering)
          assert(youtubeAnalytics.playbackStarted)
          sinon.assert.calledWith(stub, 'Video Playback Started', playback)
          sinon.assert.calledWith(
            stub,
            'Video Playback Buffer Started',
            playback
          )
        })

        it('should trigger "Playback Seek Started","Playback Seek Completed",Playback Buffer Started" if a user seeks with keyboard or seeks while video is paused', async function() {
          youtubeAnalytics.playbackStarted = true
          // simulate a keyboard seek
          youtubeAnalytics.lastRecordedTime.timeElapsed = 0
          youtubeAnalytics.lastRecordedTime.timeReported = Date.now()
          playerTime = SEEK_THRESHOLD / 1000 + 10
          window.segmentYoutubeOnStateChange({ data: YT.PlayerState.BUFFERING })

          assert(youtubeAnalytics.isBuffering)
          assert(!youtubeAnalytics.isSeeking)
          sinon.assert.calledWith(stub, 'Video Playback Seek Started', playback)
          sinon.assert.calledWith(
            stub,
            'Video Playback Seek Completed',
            playback
          )
          sinon.assert.calledWith(
            stub,
            'Video Playback Buffer Started',
            playback
          )
        })

        it('should trigger "Playback Seek Completed","Playback Buffer Started" if a seek is in progress', function() {
          youtubeAnalytics.isSeeking = true
          youtubeAnalytics.playbackStarted = true
          window.segmentYoutubeOnStateChange({ data: YT.PlayerState.BUFFERING })

          assert(youtubeAnalytics.isBuffering)
          assert(!youtubeAnalytics.isSeeking)
          sinon.assert.calledWith(
            stub,
            'Video Playback Seek Completed',
            playback
          )
          sinon.assert.calledWith(
            stub,
            'Video Playback Buffer Started',
            playback
          )
        })
      })

      describe('YT play events', function() {
        it('should trigger "Content Started" if the content hadnt started yet', function() {
          window.segmentYoutubeOnStateChange({ data: YT.PlayerState.PLAYING })

          assert(youtubeAnalytics.contentStarted)
          sinon.assert.calledWith(stub, 'Video Content Started', content)
        })

        it('should trigger "Playback Buffer Completed" if a buffer is in progress', function() {
          youtubeAnalytics.contentStarted = true
          youtubeAnalytics.isBuffering = true

          window.segmentYoutubeOnStateChange({ data: YT.PlayerState.PLAYING })

          assert(!youtubeAnalytics.isBuffering)
          sinon.assert.calledWith(
            stub,
            'Video Playback Buffer Completed',
            playback
          )
        })

        it('should trigger "Playback Resumed" if the video is paused', function() {
          youtubeAnalytics.contentStarted = true
          youtubeAnalytics.isPaused = true

          window.segmentYoutubeOnStateChange({ data: YT.PlayerState.PLAYING })
          assert(!youtubeAnalytics.isPaused)
          sinon.assert.calledWith(stub, 'Video Playback Resumed', playback)
        })
      })

      describe('YT pause events', function() {
        it('should trigger "Playback Buffer" if the video was buffering', function() {
          youtubeAnalytics.isBuffering = true
          window.segmentYoutubeOnStateChange({ data: YT.PlayerState.PAUSED })

          assert(!youtubeAnalytics.isBuffering)
          sinon.assert.calledWith(
            stub,
            'Video Playback Buffer Completed',
            playback
          )
        })

        it('should trigger "Playback Seek Started" if a seek is detected', function() {
          youtubeAnalytics.lastRecordedTime.timeElapsed = 0
          youtubeAnalytics.lastRecordedTime.timeReported = Date.now()
          playerTime = playback.position = SEEK_THRESHOLD / 1000 + 10

          window.segmentYoutubeOnStateChange({ data: YT.PlayerState.PAUSED })

          assert(youtubeAnalytics.isSeeking)
          sinon.assert.calledWith(stub, 'Video Playback Seek Started', playback)
        })

        it('should trigger "Playback Paused" if the video wasnt paused and a seek wasnt detected', function() {
          window.segmentYoutubeOnStateChange({ data: YT.PlayerState.PAUSED })

          assert(youtubeAnalytics.isPaused)
          sinon.assert.calledWith(stub, 'Video Playback Paused', playback)
        })
      })

      describe('YT ended events', function() {
        it('should trigger "Video Content Completed" and "Video Playback Completed" as its the only video in the playlist', function() {
          window.segmentYoutubeOnStateChange({ data: YT.PlayerState.ENDED })

          assert(!youtubeAnalytics.contentStarted)
          sinon.assert.calledWith(stub, 'Video Content Completed', content)
          sinon.assert.calledWith(stub, 'Video Playback Completed', playback)
        })
      })
    })

    describe('Video Playlist Playback', function() {
      beforeEach(function() {
        youtubeAnalytics.metadata = [
          { playback, content },
          { playback, content }
        ]
        playlist = ['', '']
      })

      it('should set contentStarted to false if user clicks next video', function() {
        playerTime = 0
        playlistIndex = 1
        youtubeAnalytics.playlistIndex = 0

        window.segmentYoutubeOnStateChange({ data: YT.PlayerState.BUFFERING })

        assert(!youtubeAnalytics.contentStarted)
      })

      it('should trigger "Playback Completed" and then "Playback Started" when user clicks next video on the last video of playlist', function() {
        playerTime = 0
        playlistIndex = 0
        youtubeAnalytics.playlistIndex = 1
        youtubeAnalytics.playbackStarted = true

        window.segmentYoutubeOnStateChange({ data: YT.PlayerState.BUFFERING })

        assert(!youtubeAnalytics.contentStarted)
        assert(youtubeAnalytics.isBuffering)
        sinon.assert.calledWith(stub, 'Video Playback Completed', playback)
        sinon.assert.calledWith(stub, 'Video Playback Started', playback)
        sinon.assert.calledWith(stub, 'Video Playback Buffer Started', playback)
      })

      it('should trigger "Content Completed" but not "Playback Completed" when a video in the middle of the playlist ends', function() {
        playlistIndex = 0

        window.segmentYoutubeOnStateChange({ data: YT.PlayerState.ENDED })

        assert(!youtubeAnalytics.contentStarted)
        sinon.assert.calledWith(stub, 'Video Content Completed', content)
        sinon.assert.neverCalledWith(stub, 'Video Playback Completed')
      })

      it('should trigger "Content Completed" and "Playback Completed" when the final video ends', function() {
        playlistIndex = 1

        window.segmentYoutubeOnStateChange({ data: YT.PlayerState.ENDED })

        assert(!youtubeAnalytics.contentStarted)
        sinon.assert.calledWith(stub, 'Video Content Completed', content)
        sinon.assert.calledWith(stub, 'Video Playback Completed', playback)
      })
    })
  })
})

function clearEnv(id) {
  document.getElementById(id).remove()
}
